/*
								+---------------------------------+
								|                                 |
								|      ***   OpenGlDrv   ***      |
								|                                 |
								|  Copyright   -tHE SWINe- 2002  |
								|                                 |
								|          OpenGlDrv.cpp          |
								|                                 |
								+---------------------------------+
*/

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "OpenGlDrv.h"

static HDC _hDC;
static HGLRC _hRC;
static HWND _hWnd;
static int _b_Fullscreen;

static int n_width;
static int n_height;

static int n_status = 0;

// vytvori gl kontext pro nase okno
int Init_GL(HWND hWnd, int n_New_Width, int n_New_Height, int n_Bpp, int b_Fullscreen)
{
    PIXELFORMATDESCRIPTOR pixelformat;
    DEVMODE dmScreenSettings;
    GLuint PixelFormat;

    n_status = 0;

    if(b_Fullscreen) {
        memset(&dmScreenSettings, 0, sizeof(dmScreenSettings));
        dmScreenSettings.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;
        dmScreenSettings.dmSize = sizeof(dmScreenSettings);
        dmScreenSettings.dmPelsWidth = n_New_Width;
        dmScreenSettings.dmPelsHeight = n_New_Height;
        dmScreenSettings.dmBitsPerPel = n_Bpp;
        // nastavi rozliseni (pokud fullscreen)

        if(ChangeDisplaySettings(&dmScreenSettings, 4) != DISP_CHANGE_SUCCESSFUL) {
            if(MessageBox(NULL, "Unable to set fullscreen mode.\n"
               "Use windowed mode instead?","Fullscreen", MB_YESNO | MB_ICONEXCLAMATION) == IDYES)
                _b_Fullscreen = 0;
            else
                return 0;
        }
        // zkusi prepnout, pokud to nejde, zepta
        // se a pak skonci nebo zustane v okne

        ShowCursor(FALSE);
    }

    memset(&pixelformat, 0, sizeof(PIXELFORMATDESCRIPTOR));
    pixelformat.nSize = sizeof(PIXELFORMATDESCRIPTOR);
    pixelformat.nVersion = 1;
    pixelformat.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
    pixelformat.dwLayerMask = PFD_MAIN_PLANE;
    pixelformat.iPixelType = PFD_TYPE_RGBA;
    pixelformat.cColorBits = n_Bpp;
    pixelformat.cDepthBits = n_Bpp;
    pixelformat.cAccumBits = 0;
    pixelformat.cStencilBits = 0;
    // nastavi pixel format descriptor

    if(!(_hDC = GetDC(hWnd))) {
        MessageBox(NULL, "Unable to create a GL device context.", "GL_Drv", MB_OK);
        return 0;
    }
    if(!(PixelFormat = ChoosePixelFormat(_hDC, &pixelformat))) {
        MessageBox(NULL, "Unable to find a suitable pixelformat.", "GL_Drv", MB_OK);
        return 0;
    }
    if(!SetPixelFormat(_hDC, PixelFormat, &pixelformat)) {
        MessageBox(NULL, "Unable to set the pixelformat.", "GL_Drv", MB_OK);
        return 0;
    }
    if(!(_hRC = wglCreateContext(_hDC))) {
        MessageBox(NULL, "Unable to create a GL rendering context.", "GL_Drv", MB_OK);
        return 0;
    }
    if(!wglMakeCurrent(_hDC, _hRC)) {
        MessageBox(NULL, "Unable to activate the GL rendering context.", "GL_Drv", MB_OK);
        return 0;
    }
    // vytvori OpenGl kontext

    glViewport(0, 0, n_New_Width, n_New_Height);
    // nastavi nase okno

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(90, (GLfloat)n_New_Width / (GLfloat)n_New_Height, 0.5, 900);
    // nastvi projekci

    n_width = n_New_Width;
    n_height = n_New_Height;
    n_status = 1;
    _b_Fullscreen = b_Fullscreen;
    // nastavi

	glDepthFunc(GL_LEQUAL);
	glCullFace(GL_FRONT);
	glEnable(GL_CULL_FACE);
	glEnable(GL_DEPTH_TEST);
	glShadeModel(GL_FLAT);
	glMatrixMode(GL_PROJECTION);
	glEnable(GL_TEXTURE_2D);

    return 1;
}

// ukonci praci s gl
int Terminate_GL()
{
    int i = 1;

    n_status = 0;

    if(_b_Fullscreen) {
        ChangeDisplaySettings(NULL, 0);
        ShowCursor(TRUE);
    }
    // pokud bylo fullscreen, vrati zpatky

    if(_hRC) {
        if(!wglMakeCurrent(NULL, NULL)) {
            MessageBox(NULL, "Release of DC and RC failed.", "GL_Drv", MB_OK);
            i = 0;
        }
        if(!wglDeleteContext(_hRC)) {
            MessageBox(NULL, "Release rendering context failed.", "GL_Drv", MB_OK);
            _hRC = NULL;
            i = 0;
        }
    }

    if(_hDC && !ReleaseDC(_hWnd, _hDC)) {
        _hDC = NULL;
        i = 0;
    }

    return i;
}

// ukonci kresleni, zobrazi vysledek
void Blit()
{
    /*glFlush();
    glFinish();*/
    SwapBuffers(_hDC);
}
